-----------------------------------------------------------------------------------------------
-- Client Lua Script for MainName
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

require "Apollo"
require "Window"
require "ChatSystemLib"
require "GameLib"

-----------------------------------------------------------------------------------------------
-- MainName Module Definition
-----------------------------------------------------------------------------------------------
local MainName = {} 

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

local releaseType = "Release"
local releaseVersion = "1.0.5"
local killroyVersion = "1.4.6"

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function MainName:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self


	 -- initialize variables here
	 -- this data will only be used if no variables have been saved to file
	if not(self.mnPrefs) then
		
		self.mnPrefs =
		{
			strName = "",
			bEnabled = false,
			bAcctWhisper = false,
			bCustom = false,
			bGuild = false,
			bInstance = false,
			bParty = false,
			bSay = false,
			bSociety = false,
			bWarParty = false,
			bWhisper = false,
			bYell = false,
			bZone = false,
			bZonePvP = false,
			cLeftWrap = '[',
			cRightWrap = ']'
		}
	end

    return o
end

function MainName:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "MainName"
	
	if Apollo.GetAddon("ImprovedChatLog") ~= nil then
		local sDep = "ImpvrovedChatLog"
	elseif Apollo.GetAddon("Killroy") ~= nil then
		local sDep = "Killroy"
	else 
		local sDep = "ChatLog"
	end
	local tDependencies = {
		sDep
	}
	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- MainName OnLoad
-----------------------------------------------------------------------------------------------
function MainName:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("MainName.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function MainName:OnDocLoaded()


    MainName.wndMain = Apollo.LoadForm("MainName.xml", "MainNameForm", nil, self)
    MainName.wndMain:Show(false, true)

	-- if the xmlDoc is no longer needed, you should set it to nil
	self.xmlDoc = nil
	
	-- Register handlers for events, slash commands and timer, etc.
	Apollo.RegisterSlashCommand("mn", "OnSlashCommand", self)
	Apollo.RegisterSlashCommand("MainName", "OnSlashCommand", self)


	-- Do additional Addon initialization here
	if Apollo.GetAddon("Killroy") ~= nil then
		MainName:Change_CVCKillroy()
	else
		MainName:Change_VerifyChannelVisibility() --Overrides the ChatLog function of the same name
	end
end

-----------------------------------------------------------------------------------------------
-- MainName Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

--Configuration Invoke
function MainName:OnConfig()
	self:FormSet()
	self.wndMain:Show(true)
end

-- on SlashCommand "/mn"
function MainName:OnSlashCommand(strCommand, strArg)

	if strArg == "" then
		 MainName:OnConfig() -- show the window
	else	
		self.mnPrefs['strName'] = strArg
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "Name set to '"..strArg.."'.", "")

	end
end

--Replaces Carbine's Change_VerifyChannelVisibility() Function from ChatLog
function MainName:Change_VerifyChannelVisibility()
	local aAddon 	

	if (Apollo.GetAddon("ChatLog") ~= nil) then
		aAddon = Apollo.GetAddon("ChatLog")
	elseif (Apollo.GetAddon("ImprovedChatLog") ~= nil) then
		aAddon = Apollo.GetAddon("ImprovedChatLog")
	else
		return nil
	end
	
	function aAddon:VerifyChannelVisibility(channelChecking, tInput, wndChat)
	
		local tChatData = wndChat:GetData()

		--if tChatData.tViewedChannels[ channelChecking:GetType() ] ~= nil then
		if MainName:checkSetting("bEnabled") == true and self.tAllViewedChannels[ channelChecking:GetType() ] ~= nil then -- see if this channelChecking is viewed
			local strMessage = tInput.strMessage
			local checkChan = channelChecking:GetType()
			if checkChan == ChatSystemLib.ChatChannel_AccountWhisper then
				if self.tAccountWhisperContex then
					local strCharacterAndRealm = self.tAccountWhisperContex.strCharacterName .. "@" .. self.tAccountWhisperContex.strRealmName
					if MainName:checkSetting("bAcctName") and strMessage ~= "" then
						strMessage = MainName:BuildName().." "..strMessage
					end
					strMessage = string.gsub(strMessage, self.tAccountWhisperContex.strDisplayName, strCharacterAndRealm, 1)
				end
			elseif MainName:VerifyChannelEnable("bWhisper", checkChan, ChatSystemLib.ChatChannel_Whisper) then
				--Parses whispers to allow input of the name
				local whisperParse = strMessage:find(" ")
				strMessage = strMessage:sub(0,whisperParse)..MainName:BuildName()..strMessage:sub(whisperParse,strMessage:len())
						
			elseif MainName:VerifyChannelEnable("bCustom", checkChan, ChatSystemLib.ChatChannel_Custom)
					or MainName:VerifyChannelEnable("bGuild", checkChan, ChatSystemLib.ChatChannel_Guild)
					or MainName:VerifyChannelEnable("bInstance", checkChan, ChatSystemLib.ChatChannel_Instance)
					or MainName:VerifyChannelEnable("bParty", checkChan, ChatSystemLib.ChatChannel_Party)
					or MainName:VerifyChannelEnable("bSay", checkChan, ChatSystemLib.ChatChannel_Say)
					or MainName:VerifyChannelEnable("bSociety", checkChan, ChatSystemLib.ChatChannel_Society)
					or MainName:VerifyChannelEnable("bWarParty", checkChan, ChatSystemLib.ChatChannel_WarParty)
					or MainName:VerifyChannelEnable("bYell", checkChan, ChatSystemLib.ChatChannel_Yell) 
					or MainName:VerifyChannelEnable("bZone", checkChan, ChatSystemLib.ChatChannel_Zone)
					or MainName:VerifyChannelEnable("bZonePvP", checkChan, ChatSystemLib.ChatChannel_ZonePvP) then
				
				if strMessage ~= "" then
					strMessage = MainName:BuildName().." "..strMessage
				end
			end

			channelChecking:Send(strMessage)
			
			return true
		elseif self.tAllViewedChannels[ channelChecking:GetType() ] ~= nil then -- see if this channelChecking is viewed
			local strMessage = tInput.strMessage
			if channelChecking:GetType() == ChatSystemLib.ChatChannel_AccountWhisper then
				if self.tAccountWhisperContex then
					local strCharacterAndRealm = self.tAccountWhisperContex.strCharacterName .. "@" .. self.tAccountWhisperContex.strRealmName
					strMessage = string.gsub(strMessage, self.tAccountWhisperContex.strDisplayName, strCharacterAndRealm, 1)
				end
			end
			channelChecking:Send(strMessage)
			return true
		else
			local wndInput = wndChat:FindChild("Input")
	
			strMessage = String_GetWeaselString(Apollo.GetString("CRB_Message_not_sent_you_are_not_viewing"), channelChecking:GetName())
			ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_Command, strMessage, "" )
			wndInput:SetText(String_GetWeaselString(Apollo.GetString("ChatLog_MessageToPlayer"), tInput.strCommand, tInput.strMessage))
			wndInput:SetFocus()
			local strSubmitted = wndInput:GetText()
			wndInput:SetSel(strSubmitted:len(), -1)
			return false
		end
	end
end


function MainName:Change_CVCKillroy()
	if (Apollo.GetAddon("ChatLog") ~= nil) then
		aAddon = Apollo.GetAddon("ChatLog")
	else
		return nil
	end

	function aAddon:VerifyChannelVisibility(channelChecking, tInput, wndChat)
		Killroy = Apollo.GetAddon("Killroy")
			if not Killroy then 
				MainName:Change_VerifyChannelVisibility()
				return
			end	

		local tChatData = wndChat:GetData()
	
		--if tChatData.tViewedChannels[ channelChecking:GetType() ] ~= nil then
		--use cludge if custom colors enabled
		local nTestChannelType
		if Killroy.tPrefs["bCustomChatColors"] then
			nTestChannelType = self.tAllViewedChannels[ Killroy:ChannelCludge(channelChecking:GetName(), channelChecking:GetType()) ]
		else
			nTestChannelType = self.tAllViewedChannels[ channelChecking:GetType() ]
		end
		
		if MainName:checkSetting("bEnabled") == true and nTestChannelType ~= nil then -- see if this channelChecking is viewed
			local strMessage = tInput.strMessage
			
			--as previous, use cludge if
			local nCheckingType
			if Killroy.tPrefs["bCustomChatColors"] then
				nCheckingType = Killroy:ChannelCludge(channelChecking:GetName(),channelChecking:GetType())
			else
				nCheckingType = channelChecking:GetType()
			end
			
			if nCheckingType == ChatSystemLib.ChatChannel_AccountWhisper then
				if self.tAccountWhisperContex then
					local strCharacterAndRealm = self.tAccountWhisperContex.strCharacterName .. "@" .. self.tAccountWhisperContex.strRealmName
					if MainName:checkSetting("bAcctName") and strMessage ~= "" then
						strMessage = MainName:BuildName().." "..strMessage
					end
					strMessage = string.gsub(strMessage, self.tAccountWhisperContex.strDisplayName, strCharacterAndRealm, 1)
				end
			elseif MainName:VerifyChannelEnable("bWhisper", nCheckingType, ChatSystemLib.ChatChannel_Whisper) then
				--Parses whispers to allow input of the name
				
				local whisperParse = strMessage:find(" ")
				strMessage = strMessage:sub(0,whisperParse)..MainName:BuildName()..strMessage:sub(whisperParse,strMessage:len())
						
			elseif MainName:VerifyChannelEnable("bCustom", nCheckingType, ChatSystemLib.ChatChannel_Custom)
					or MainName:VerifyChannelEnable("bGuild", nCheckingType, ChatSystemLib.ChatChannel_Guild)
					or MainName:VerifyChannelEnable("bInstance", nCheckingType, ChatSystemLib.ChatChannel_Instance)
					or MainName:VerifyChannelEnable("bParty", nCheckingType, ChatSystemLib.ChatChannel_Party)
					or MainName:VerifyChannelEnable("bSay", nCheckingType, ChatSystemLib.ChatChannel_Say)
					or MainName:VerifyChannelEnable("bSociety", nCheckingType, ChatSystemLib.ChatChannel_Society)
					or MainName:VerifyChannelEnable("bWarParty", nCheckingType, ChatSystemLib.ChatChannel_WarParty)
					or MainName:VerifyChannelEnable("bYell", nCheckingType, ChatSystemLib.ChatChannel_Yell) 
					or MainName:VerifyChannelEnable("bZone", nCheckingType, ChatSystemLib.ChatChannel_Zone)
					or MainName:VerifyChannelEnable("bZonePvP", nCheckingType, ChatSystemLib.ChatChannel_ZonePvP) then
				
				if strMessage ~= "" then
					strMessage = MainName:BuildName().." "..strMessage
				end
			end
		
			-- filter for targets and embedded emotes before sending to channel
			local strTargetFiltered = Killroy:ParseForTarget(strMessage)
			
			-- adding a check for inline emotes to say and emote only
			local bEmoteOrSay = (nCheckingType == ChatSystemLib.ChatChannel_Emote) or (nCheckingType == ChatSystemLib.ChatChannel_Say)
			local tEmoteFiltered, strEmbeddedEmote
			if bEmoteOrSay then
				tEmoteFiltered = Killroy:ParseForAnimatedEmote(strTargetFiltered)
				strEmbeddedEmote = tEmoteFiltered["strEmbeddedEmote"]
			end

			strMessage = strTargetFiltered
			--break up sends of more than 500 chars
			if string.len(strMessage)>500 then
				local tChunks = {}
				local pattern = "%s*[^%s]+%s*"
				local nLength = 0
				local strCompile = ""
				for this_word in string.gmatch(strMessage, pattern) do
					nLength = nLength + string.len(this_word)
					if nLength > 480 then
						table.insert(tChunks, strCompile)
						nLength = 0
						strCompile = ""
					end
					strCompile = strCompile..this_word 
				end
				table.insert(tChunks, strCompile)
				
				for i, this_chunk in ipairs(tChunks) do
					channelChecking:Send(this_chunk)
				end
			else
			channelChecking:Send(strMessage)
			end
			
			if strEmbeddedEmote then
				local ComChan = Killroy:GetChannelByName("Command")
				ComChan:Send(strEmbeddedEmote)
			end
			
			return true
		elseif nTestChannelType ~= nil then -- see if this channelChecking is viewed
			local strMessage = tInput.strMessage
			
			--as previous, use cludge if
			local nCheckingType
			if Killroy.tPrefs["bCustomChatColors"] then
				nCheckingType = Killroy:ChannelCludge(channelChecking:GetName(),channelChecking:GetType())
			else
				nCheckingType = channelChecking:GetType()
			end
			
			if nCheckingType == ChatSystemLib.ChatChannel_AccountWhisper then
				if self.tAccountWhisperContex then
					local strCharacterAndRealm = self.tAccountWhisperContex.strCharacterName .. "@" .. self.tAccountWhisperContex.strRealmName
					strMessage = string.gsub(strMessage, self.tAccountWhisperContex.strDisplayName, strCharacterAndRealm, 1)
				end
			end
			
			-- filter for targets and embedded emotes before sending to channel
			local strTargetFiltered = Killroy:ParseForTarget(strMessage)
			
			-- adding a check for inline emotes to say and emote only
			local bEmoteOrSay = (nCheckingType == ChatSystemLib.ChatChannel_Emote) or (nCheckingType == ChatSystemLib.ChatChannel_Say)
			local tEmoteFiltered, strEmbeddedEmote
			if bEmoteOrSay then
				tEmoteFiltered = Killroy:ParseForAnimatedEmote(strTargetFiltered)
				strEmbeddedEmote = tEmoteFiltered["strEmbeddedEmote"]
			end

			strMessage = strTargetFiltered
			--break up sends of more than 500 chars
			if string.len(strMessage)>500 then
				local tChunks = {}
				local pattern = "%s*[^%s]+%s*"
				local nLength = 0
				local strCompile = ""
				for this_word in string.gmatch(strMessage, pattern) do
					nLength = nLength + string.len(this_word)
					if nLength > 480 then
						table.insert(tChunks, strCompile)
						nLength = 0
						strCompile = ""
					end
					strCompile = strCompile..this_word 
				end
				table.insert(tChunks, strCompile)
				
				for i, this_chunk in ipairs(tChunks) do
					channelChecking:Send(this_chunk)
				end
			else
			channelChecking:Send(strMessage)
			end
			
			if strEmbeddedEmote then
				local ComChan = Killroy:GetChannelByName("Command")
				ComChan:Send(strEmbeddedEmote)
			end
			
			return true
		else
			local wndInput = wndChat:FindChild("Input")
	
			strMessage = String_GetWeaselString(Apollo.GetString("CRB_Message_not_sent_you_are_not_viewing"), channelChecking:GetName())
			ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_Command, strMessage, "" )
			wndInput:SetText(String_GetWeaselString(Apollo.GetString("ChatLog_MessageToPlayer"), tInput.strCommand, tInput.strMessage))
			wndInput:SetFocus()
			local strSubmitted = wndInput:GetText()
			wndInput:SetSel(strSubmitted:len(), -1)
			return false
		end
	end
end

--checks to see if MN is enabled for a specific channel.
function MainName:VerifyChannelEnable(strSetting, checkChan, cslSetting)
	if self:checkSetting(strSetting) and checkChan == cslSetting then
		return true
	else
		return false
	end
end

--Checks to see if a MN setting is on or off.
function MainName:checkSetting(strSetting)
	if self.mnPrefs[strSetting] == true then
		return true
	elseif self.mnPrefs[strSetting] == false then
		return false
	end
end

function MainName:checkName()
	return self.mnPrefs['strName']
end

function MainName:checkWrap(iWrapType)
	if iWrapType == 1 then
		return self.mnPrefs['cLeftWrap']
	elseif iWrapType == 2 then
		return self.mnPrefs['cRightWrap']
	end
end

function MainName:BuildName()
	return (self:checkWrap(1)..self:checkName()..self:checkWrap(2))
end

-----------------------------------------------------------------------------------------------
-- MainNameForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function MainName:OnOK()
	self.mnPrefs["bEnabled"] = MainName.wndMain:FindChild("MNEnabled"):IsChecked()
	self.mnPrefs["strName"] = MainName.wndMain:FindChild("MNNameBox"):GetText()
	self.mnPrefs["bCustom"] = MainName.wndMain:FindChild("MNCustom"):IsChecked()
	self.mnPrefs["bGuild"] = MainName.wndMain:FindChild("MNGuild"):IsChecked()
	self.mnPrefs["bInstance"] = MainName.wndMain:FindChild("MNInstance"):IsChecked()
	self.mnPrefs["bParty"] = MainName.wndMain:FindChild("MNParty"):IsChecked()
	self.mnPrefs["bSay"] = MainName.wndMain:FindChild("MNSay"):IsChecked()
	self.mnPrefs["bSociety"] = MainName.wndMain:FindChild("MNSociety"):IsChecked()
	self.mnPrefs["bWarParty"] = MainName.wndMain:FindChild("MNWarParty"):IsChecked()
	self.mnPrefs["bWhisper"] = MainName.wndMain:FindChild("MNWhisper"):IsChecked()
	self.mnPrefs["bYell"] = MainName.wndMain:FindChild("MNYell"):IsChecked()
	self.mnPrefs["bZone"] = MainName.wndMain:FindChild("MNZone"):IsChecked()
	self.mnPrefs["bZonePvP"] = MainName.wndMain:FindChild("MNZonePvP"):IsChecked()
	
	local WrapBoxText = self.wndMain:FindChild("MNWrapBox"):GetText()
	if WrapBoxText:len() <= 1 then
		self.mnPrefs["cLeftWrap"] = WrapBoxText
		self.mnPrefs["cRightWrap"] = WrapBoxText
	elseif WrapBoxText:len() == 2 then
		self.mnPrefs["cLeftWrap"] = WrapBoxText:sub(1,1)
		self.mnPrefs["cRightWrap"] = WrapBoxText:sub(2,2)
	end
	
	MainName.wndMain:Close() -- hide the window
end

function MainName:FormSet()
	if MainName:checkSetting("bEnabled") then
		self.wndMain:FindChild("MNEnabled"):SetCheck(true)
		self.wndMain:FindChild("MNDisabled"):SetCheck(false)
	else
		self.wndMain:FindChild("MNEnabled"):SetCheck(false)
		self.wndMain:FindChild("MNDisabled"):SetCheck(true)
	end
	self.wndMain:FindChild("MNNameBox"):SetText(self:checkName("strName"))
	self.wndMain:FindChild("MNAcctWhisper"):SetCheck(self:checkSetting("bAcctWhisper"))
	self.wndMain:FindChild("MNCustom"):SetCheck(self:checkSetting("bCustom"))
	self.wndMain:FindChild("MNGuild"):SetCheck(self:checkSetting("bGuild"))
	self.wndMain:FindChild("MNInstance"):SetCheck(self:checkSetting("bInstance"))
	self.wndMain:FindChild("MNParty"):SetCheck(self:checkSetting("bParty"))
	self.wndMain:FindChild("MNSay"):SetCheck(self:checkSetting("bSay"))
	self.wndMain:FindChild("MNSociety"):SetCheck(self:checkSetting("bSociety"))
	self.wndMain:FindChild("MNWarParty"):SetCheck(self:checkSetting("bWarParty"))
	self.wndMain:FindChild("MNWhisper"):SetCheck(self:checkSetting("bWhisper"))
	self.wndMain:FindChild("MNYell"):SetCheck(self:checkSetting("bYell"))
	self.wndMain:FindChild("MNZone"):SetCheck(self:checkSetting("bZone"))
	self.wndMain:FindChild("MNZonePvP"):SetCheck(self:checkSetting("bZonePvP"))
	if self:checkWrap(1) ~= self:checkWrap(2) then
		self.wndMain:FindChild("MNWrapBox"):SetText(self:checkWrap(1)..self:checkWrap(2))
	else
		self.wndMain:FindChild("MNWrapBox"):SetText(self:checkWrap(1))
	end
end

-- when the Cancel button is clicked
function MainName:OnCancel()
	self.wndMain:Close() -- hide the window
	self:FormSet()

end

function MainName:WrapChanged( wndHandler, wndControl, strText )
	if Apollo.GetAddon("Killroy") then
		if (strText:sub(1,1) == '{' or strText:sub(1,1) == '}')
		  or (strText:sub(2,2) == '{' or strText:sub(2,2) == '}') then
			self.wndMain:FindChild("MNWrapBox"):SetText("")
		end
	end
	if strText:len() > 2 then
		self.wndMain:FindChild("MNWrapBox"):SetText(strText:sub(1,2))
	end
end

-----------------------------------------------------------------------------------------------
-- Save and Restore
-----------------------------------------------------------------------------------------------
function MainName:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Realm then
        return nil
    end

	return {mnPrefs = self.mnPrefs}
end

function MainName:OnRestore(eLevel,tData)
	--Set Preferences from File
	if (tData.mnPrefs ~= nil) then
		for x,n in pairs(tData.mnPrefs) do
			self.mnPrefs[x] = n
		end
	end
end

-----------------------------------------------------------------------------------------------
-- MainName Instance
-----------------------------------------------------------------------------------------------
local MainNameInst = MainName:new()
MainNameInst:Init()
